@@ -1,6 +1,9 @@
 <?php
-/* Copyright (C) 2011-2022	Regis Houssin		<regis.houssin@inodbox.com>
- * Copyright (C) 2011-2012	Laurent Destailleur	<eldy@users.sourceforge.net>
+
+/* Copyright (C) 2011-2022	Regis Houssin			<regis.houssin@inodbox.com>
+ * Copyright (C) 2011-2023	Laurent Destailleur		<eldy@users.sourceforge.net>
+ * Copyright (C) 2024		Frédéric France			<frederic.france@free.fr>
+ * Copyright (C) 2024-2025	MDW						<mdeweerd@users.noreply.github.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +20,10 @@
  */
 
 /**
- *       \file       htdocs/core/class/fileupload.class.php
- *       \brief      File to return Ajax response on file upload
+ *       \file      htdocs/core/class/fileupload.class.php
+ *       \brief     File to return the ajax response of core/ajax/fileupload.php for common file upload.
+ *       			Security is check by the ajax component.
+ *       			For large files upload, see flowjs-server.php
  */
 
 require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
@@ -30,21 +35,31 @@ require_once DOL_DOCUMENT_ROOT.'/core/lib/images.lib.php';
  */
 class FileUpload
 {
+	/**
+	 * @var array{script_url:string,upload_dir:string,upload_url:string,param_name:string,delete_type:string,max_file_size:?int,min_file_size:int,accept_file_types:string,max_number_of_files:?int,max_width:?int,max_height:?int,min_width:int,min_height:int,discard_aborted_uploads:bool,image_versions:array<string,array{upload_dir:string,upload_url:string,max_width:int,max_height:int,jpeg_quality?:int}>}
+	 */
 	public $options;
+	/**
+	 * @var int
+	 */
 	protected $fk_element;
+
+	/**
+	 * @var string object element
+	 */
 	protected $element;
 
 	/**
 	 * Constructor.
 	 * This set ->$options
 	 *
-	 * @param array		$options		Options array
+	 * @param ?array{script_url?:string,upload_dir?:string,upload_url?:string,param_name?:string,delete_type?:string,max_file_size?:?int,min_file_size?:int,accept_file_types?:string,max_number_of_files?:?int,max_width?:?int,max_height?:?int,min_width?:int,min_height?:int,discard_aborted_uploads?:bool,image_versions?:array<string,array{upload_dir?:string,upload_url?:string,max_width?:int,max_height?:int,jpeg_quality?:int}>}	$options		Options array
 	 * @param int		$fk_element		ID of element
 	 * @param string	$element		Code of element
 	 */
 	public function __construct($options = null, $fk_element = null, $element = null)
 	{
-		global $db, $conf;
+		global $db;
 		global $hookmanager;
 
 		$hookmanager->initHooks(array('fileupload'));
@@ -56,17 +71,17 @@ class FileUpload
 		$this->element = $element;
 
 		$pathname = str_replace('/class', '', $element_prop['classpath']);
-		$filename = $element_prop['classfile'];
-		$dir_output = $element_prop['dir_output'];
-		$savingDocMask = '';
+		$filename = dol_sanitizeFileName($element_prop['classfile']);
+		$dir_output = dol_sanitizePathName($element_prop['dir_output']);
 
 		//print 'fileupload.class.php: element='.$element.' pathname='.$pathname.' filename='.$filename.' dir_output='.$dir_output."\n";
 
 		if (empty($dir_output)) {
-			setEventMessage('The element '.$element.' is not supported for uploading file. dir_output is unknow.', 'errors');
-			throw new Exception('The element '.$element.' is not supported for uploading file. dir_output is unknow.');
+			setEventMessage('The element '.$element.' is not supported for uploading file. dir_output is unknown.', 'errors');
+			throw new Exception('The element '.$element.' is not supported for uploading file. dir_output is unknown.');
 		}
 
+		$object_ref = 'UndefinedReference';
 		// If pathname and filename are null then we can still upload files if we have specified upload_dir on $options
 		if ($pathname !== null && $filename !== null) {
 			// Get object from its id and type
@@ -74,11 +89,6 @@ class FileUpload
 
 			$object_ref = dol_sanitizeFileName($object->ref);
 
-			// add object reference as file name prefix if const MAIN_DISABLE_SUGGEST_REF_AS_PREFIX is not enabled
-			if (!getDolGlobalInt('MAIN_DISABLE_SUGGEST_REF_AS_PREFIX')) {
-				$savingDocMask = $object_ref.'-__file__';
-			}
-
 			// Special cases to forge $object_ref used to forge $upload_dir
 			if ($element == 'invoice_supplier') {
 				$object_ref = get_exdir($object->id, 2, 0, 0, $object, 'invoice_supplier').$object_ref;
@@ -104,7 +114,6 @@ class FileUpload
 			'script_url' => $_SERVER['PHP_SELF'],
 			'upload_dir' => $dir_output.'/'.$object_ref.'/',
 			'upload_url' => DOL_URL_ROOT.'/document.php?modulepart='.$element.'&attachment=1&file=/'.$object_ref.'/',
-			'saving_doc_mask' => $savingDocMask,
 			'param_name' => 'files',
 			// Set the following option to 'POST', if your server does not support
 			// DELETE requests. This is a parameter sent to the client:
@@ -138,7 +147,7 @@ class FileUpload
 				*/
 				'thumbnail' => array(
 					'upload_dir' => $dir_output.'/'.$object_ref.'/thumbs/',
-					'upload_url' => DOL_URL_ROOT.'/document.php?modulepart='.urlencode($element).'&attachment=1&file=/'.$object_ref.'/thumbs/',
+					'upload_url' => DOL_URL_ROOT.'/document.php?modulepart='.urlencode($element).'&attachment=1&file='.urlencode('/'.$object_ref.'/thumbs/'),
 					'max_width' => 80,
 					'max_height' => 80
 				)
@@ -153,7 +162,7 @@ class FileUpload
 				'options' => &$options,
 				'element' => $element
 			),
-			$object,
+			$object,  // @phan-suppress-current-line PhanTypeMismatchArgumentNullable
 			$action
 		);
 
@@ -161,10 +170,9 @@ class FileUpload
 			$this->options = array_replace_recursive($this->options, $options);
 		}
 
-		// At this point we should have a valid upload_dir in options
-		//if ($pathname === null && $filename === null) { // OR or AND???
-		if ($pathname === null || $filename === null) {
-			if (!key_exists("upload_dir", $this->options)) {
+		// At this point we should have a valid upload_dir in this->options
+		if (empty($pathname) || empty($filename)) {
+			if (!array_key_exists("upload_dir", $this->options)) {
 				setEventMessage('If $fk_element = null or $element = null you must specify upload_dir on $options', 'errors');
 				throw new Exception('If $fk_element = null or $element = null you must specify upload_dir on $options');
 			} elseif (!is_dir($this->options['upload_dir'])) {
@@ -197,13 +205,12 @@ class FileUpload
 	/**
 	 * Set delete url
 	 *
-	 * @param 	object	$file		Filename
+	 * @param 	stdClass	$file		File object (see getFileObject)
 	 * @return	void
 	 */
 	protected function setFileDeleteUrl($file)
 	{
-		$file->delete_url = $this->options['script_url']
-		.'?file='.urlencode($file->name).'&fk_element='.urlencode($this->fk_element).'&element='.urlencode($this->element);
+		$file->delete_url = $this->options['script_url'].'?file='.urlencode((string) ($file->name)).'&fk_element='.urlencode((string) ($this->fk_element)).'&element='.urlencode((string) ($this->element));
 		$file->delete_type = $this->options['delete_type'];
 		if ($file->delete_type !== 'DELETE') {
 			$file->delete_url .= '&_method=DELETE';
@@ -214,25 +221,26 @@ class FileUpload
 	 * getFileObject
 	 *
 	 * @param	string		$file_name		Filename
-	 * @return 	stdClass|null
+	 * @return 	?stdClass
 	 */
 	protected function getFileObject($file_name)
 	{
+		$file_path = $this->options['upload_dir'].dol_sanitizeFileName($file_name);
 
-		$file_path = $this->options['upload_dir'].$file_name;
-		if (is_file($file_path) && $file_name[0] !== '.') {
+		if (dol_is_file($file_path) && $file_name[0] !== '.') {
 			$file = new stdClass();
 			$file->name = $file_name;
 			$file->mime = dol_mimetype($file_name, '', 2);
 			$file->size = filesize($file_path);
-			$file->url = $this->options['upload_url'].rawurlencode($file->name);
+			$file->url = $this->options['upload_url'].urlencode($file->name);
+
 			foreach ($this->options['image_versions'] as $version => $options) {
-				if (is_file($options['upload_dir'].$file_name)) {
+				if (dol_is_file($options['upload_dir'].$file_name)) {
 					$tmp = explode('.', $file->name);
 
 					// We save the path of mini file into file->... (seems not used)
 					$keyforfile = $version.'_url';
-					$file->$keyforfile = $options['upload_url'].rawurlencode($tmp[0].'_mini.'.$tmp[1]);
+					$file->$keyforfile = $options['upload_url'].urlencode($tmp[0].'_mini.'.$tmp[1]);
 				}
 			}
 			$this->setFileDeleteUrl($file);
@@ -244,7 +252,7 @@ class FileUpload
 	/**
 	 * getFileObjects
 	 *
-	 * @return	array	Array of objects
+	 * @return	array<?stdClass>	Array of objects
 	 */
 	protected function getFileObjects()
 	{
@@ -254,9 +262,9 @@ class FileUpload
 	/**
 	 *  Create thumbs of a file uploaded.
 	 *
-	 *  @param	string	$file_name		Filename
-	 *  @param	string	$options 		is array('max_width', 'max_height')
-	 *  @return	boolean
+	 *  @param	string	$file_name			Filename
+	 *  @param	array{upload_dir:string}	$options 	is array('max_width', 'max_height')
+	 *  @return	bool
 	 */
 	protected function createScaledImage($file_name, $options)
 	{
@@ -288,9 +296,9 @@ class FileUpload
 	}
 
 	/**
-	 * Enter description here ...
+	 * Make validation on an uploaded file
 	 *
-	 * @param 	string	$uploaded_file		Uploade file
+	 * @param 	string	$uploaded_file		Upload file
 	 * @param 	object	$file				File
 	 * @param 	string	$error				Error
 	 * @param	string	$index				Index
@@ -311,13 +319,14 @@ class FileUpload
 			return false;
 		}
 		if ($uploaded_file && is_uploaded_file($uploaded_file)) {
-			$file_size = filesize($uploaded_file);
+			$file_size = dol_filesize($uploaded_file);
 		} else {
 			$file_size = $_SERVER['CONTENT_LENGTH'];
 		}
 		if ($this->options['max_file_size'] && (
-				$file_size > $this->options['max_file_size'] ||
-				$file->size > $this->options['max_file_size'])
+			$file_size > $this->options['max_file_size'] ||
+				$file->size > $this->options['max_file_size']
+		)
 		) {
 			$file->error = 'maxFileSize';
 			return false;
@@ -328,7 +337,8 @@ class FileUpload
 			return false;
 		}
 		if (is_numeric($this->options['max_number_of_files']) && (
-				count($this->getFileObjects()) >= $this->options['max_number_of_files'])
+			count($this->getFileObjects()) >= $this->options['max_number_of_files']
+		)
 		) {
 			$file->error = 'maxNumberOfFiles';
 			return false;
@@ -385,15 +395,14 @@ class FileUpload
 	{
 		// Remove path information and dots around the filename, to prevent uploading
 		// into different directories or replacing hidden system files.
-		// Also remove control characters and spaces (\x00..\x20) around the filename:
-		$file_name = trim(basename(stripslashes($name)), ".\x00..\x20");
+		$file_name = basename(dol_sanitizeFileName($name));
 		// Add missing file extension for known image types:
 		$matches = array();
 		if (strpos($file_name, '.') === false && preg_match('/^image\/(gif|jpe?g|png)/', $type, $matches)) {
 			$file_name .= '.'.$matches[1];
 		}
 		if ($this->options['discard_aborted_uploads']) {
-			while (is_file($this->options['upload_dir'].$file_name)) {
+			while (dol_is_file($this->options['upload_dir'].$file_name)) {
 				$file_name = $this->upcountName($file_name);
 			}
 		}
@@ -404,13 +413,14 @@ class FileUpload
 	 * handleFileUpload.
 	 * Validate data, move the uploaded file then create the thumbs if this is an image.
 	 *
-	 * @param 	string		$uploaded_file		Uploade file
+	 * @param 	string		$uploaded_file		Upload file
 	 * @param 	string		$name				Name
 	 * @param 	int			$size				Size
 	 * @param 	string		$type				Type
 	 * @param 	string		$error				Error
 	 * @param	string		$index				Index
 	 * @return stdClass|null
+	 * @see dol_add_file_process()
 	 */
 	protected function handleFileUpload($uploaded_file, $name, $size, $type, $error, $index)
 	{
@@ -420,20 +430,17 @@ class FileUpload
 		$file->size = intval($size);
 		$file->type = $type;
 
+		// Sanitize to avoid stream execution when calling file_size(). Not that this is a second security because
+		// most streams are already disabled by stream_wrapper_unregister() in filefunc.inc.php
+		$uploaded_file = preg_replace('/\s*(http|ftp|sftp|)s?:/i', '', $uploaded_file);
+		$uploaded_file = realpath($uploaded_file);	// A hack to be sure the file point to an existing file on disk (and is not a SSRF attack)
+
 		$validate = $this->validate($uploaded_file, $file, $error, $index);
 
 		if ($validate) {
 			if (dol_mkdir($this->options['upload_dir']) >= 0) {
-				// add object reference as file name prefix if const MAIN_DISABLE_SUGGEST_REF_AS_PREFIX is not enabled
-				$fileNameWithoutExt = preg_replace('/\.[^\.]+$/', '', $file->name);
-				$savingDocMask = $this->options['saving_doc_mask'];
-				if ($savingDocMask && strpos($savingDocMask, $fileNameWithoutExt) !== 0) {
-					$fileNameWithPrefix = preg_replace('/__file__/', $file->name, $savingDocMask);
-					$file->name = $fileNameWithPrefix;
-				}
-
-				$file_path = $this->options['upload_dir'].$file->name;
-				$append_file = !$this->options['discard_aborted_uploads'] && is_file($file_path) && $file->size > filesize($file_path);
+				$file_path = dol_sanitizePathName($this->options['upload_dir']).dol_sanitizeFileName($file->name);
+				$append_file = !$this->options['discard_aborted_uploads'] && dol_is_file($file_path) && $file->size > dol_filesize($file_path);
 
 				clearstatcache();
 
@@ -442,22 +449,23 @@ class FileUpload
 					if ($append_file) {
 						file_put_contents($file_path, fopen($uploaded_file, 'r'), FILE_APPEND);
 					} else {
+						// TODO Replace this with a call of dol_add_file_process(... $mode=1)
 						$result = dol_move_uploaded_file($uploaded_file, $file_path, 1, 0, 0, 0, 'userfile');
 					}
 				} else {
 					// Non-multipart uploads (PUT method support)
 					file_put_contents($file_path, fopen('php://input', 'r'), $append_file ? FILE_APPEND : 0);
 				}
-				$file_size = filesize($file_path);
+				$file_size = dol_filesize($file_path);
 				if ($file_size === $file->size) {
-					$file->url = $this->options['upload_url'].rawurlencode($file->name);
+					$file->url = $this->options['upload_url'].urlencode($file->name);
 					foreach ($this->options['image_versions'] as $version => $options) {
 						if ($this->createScaledImage($file->name, $options)) {	// Creation of thumbs mini and small is ok
 							$tmp = explode('.', $file->name);
 
 							// We save the path of mini file into file->... (seems not used)
 							$keyforfile = $version.'_url';
-							$file->$keyforfile = $options['upload_url'].rawurlencode($tmp[0].'_mini.'.$tmp[1]);
+							$file->$keyforfile = $options['upload_url'].urlencode($tmp[0].'_mini.'.$tmp[1]);
 						}
 					}
 				} elseif ($this->options['discard_aborted_uploads']) {
@@ -481,18 +489,19 @@ class FileUpload
 	 *
 	 * @return	void
 	 */
-	public function get()
+	/*public function get()
 	{
-		$file_name = isset($_REQUEST['file']) ?
-		basename(stripslashes($_REQUEST['file'])) : null;
+		$file_name = isset($_REQUEST['file']) ? basename(stripslashes($_REQUEST['file'])) : null;
 		if ($file_name) {
 			$info = $this->getFileObject($file_name);
 		} else {
 			$info = $this->getFileObjects();
 		}
+
 		header('Content-type: application/json');
 		echo json_encode($info);
 	}
+	*/
 
 	/**
 	 * Output data
@@ -503,13 +512,8 @@ class FileUpload
 	{
 		$error = 0;
 
-		if (isset($_REQUEST['_method']) && $_REQUEST['_method'] === 'DELETE') {
-			return $this->delete();
-		}
-		//var_dump($_FILES);
+		$upload = isset($_FILES[$this->options['param_name']]) ? $_FILES[$this->options['param_name']] : null;
 
-		$upload = isset($_FILES[$this->options['param_name']]) ?
-		$_FILES[$this->options['param_name']] : null;
 		$info = array();
 		if ($upload && is_array($upload['tmp_name'])) {
 			// param_name is an array identifier like "files[]",
@@ -521,7 +525,7 @@ class FileUpload
 					isset($_SERVER['HTTP_X_FILE_SIZE']) ? $_SERVER['HTTP_X_FILE_SIZE'] : $upload['size'][$index],
 					isset($_SERVER['HTTP_X_FILE_TYPE']) ? $_SERVER['HTTP_X_FILE_TYPE'] : $upload['type'][$index],
 					$upload['error'][$index],
-					$index
+					(string) $index
 				);
 				if (!empty($tmpres->error)) {
 					$error++;
@@ -537,7 +541,7 @@ class FileUpload
 				isset($_SERVER['HTTP_X_FILE_SIZE']) ? $_SERVER['HTTP_X_FILE_SIZE'] : (isset($upload['size']) ? $upload['size'] : null),
 				isset($_SERVER['HTTP_X_FILE_TYPE']) ? $_SERVER['HTTP_X_FILE_TYPE'] : (isset($upload['type']) ? $upload['type'] : null),
 				isset($upload['error']) ? $upload['error'] : null,
-				0
+				'0'
 			);
 			if (!empty($tmpres->error)) {
 				$error++;
@@ -551,7 +555,7 @@ class FileUpload
 		/* disabled. Param redirect seems not used
 		$redirect = isset($_REQUEST['redirect']) ? stripslashes($_REQUEST['redirect']) : null;
 		if ($redirect) {
-			header('Location: '.sprintf($redirect, rawurlencode($json)));
+			header('Location: '.sprintf($redirect, urlencode($json)));
 			return;
 		}
 		*/
@@ -569,18 +573,19 @@ class FileUpload
 	/**
 	 * Delete uploaded file
 	 *
+	 * @param	string	$file	File
 	 * @return	int
 	 */
-	public function delete()
+	/*
+	public function delete($file)
 	{
-		$file_name = isset($_REQUEST['file']) ?
-		basename(stripslashes($_REQUEST['file'])) : null;
-		$file_path = $this->options['upload_dir'].$file_name;
-		$success = is_file($file_path) && $file_name[0] !== '.' && unlink($file_path);
+		$file_name = $file ? basename($file) : null;
+		$file_path = $this->options['upload_dir'].dol_sanitizeFileName($file_name);
+		$success = dol_is_file($file_path) && $file_name[0] !== '.' && unlink($file_path);
 		if ($success) {
 			foreach ($this->options['image_versions'] as $version => $options) {
 				$file = $options['upload_dir'].$file_name;
-				if (is_file($file)) {
+				if (dol_is_file($file)) {
 					unlink($file);
 				}
 			}
@@ -591,4 +596,5 @@ class FileUpload
 
 		return 0;
 	}
+	*/
 }
