@@ -1,4 +1,5 @@
 <?php
+
 /* Copyright (C) 2003-2007 Rodolphe Quiedeville  <rodolphe@quiedeville.org>
  * Copyright (C) 2004-2010 Laurent Destailleur   <eldy@users.sourceforge.net>
  * Copyright (C) 2005      Marc Barilley / Ocebo <marc@ocebo.com>
@@ -7,6 +8,8 @@
  * Copyright (C) 2013      Florian Henry          <florian.henry@open-concept.pro>
  * Copyright (C) 2013      Cédric Salvador       <csalvador@gpcsolutions.fr>
  * Copyright (C) 2017      Ferran Marcet       	 <fmarcet@2byte.es>
+ * Copyright (C) 2024       Frédéric France         <frederic.france@free.fr>
+ * Copyright (C) 2025		MDW						<mdeweerd@users.noreply.github.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +28,7 @@
 /**
  *       \file       htdocs/product/document.php
  *       \ingroup    product
- *       \brief      Page des documents joints sur les produits
+ *       \brief      Page of documents attached to products/services
  */
 
 
@@ -36,17 +39,24 @@ require_once DOL_DOCUMENT_ROOT.'/product/class/product.class.php';
 require_once DOL_DOCUMENT_ROOT.'/core/lib/files.lib.php';
 require_once DOL_DOCUMENT_ROOT.'/core/lib/images.lib.php';
 require_once DOL_DOCUMENT_ROOT.'/core/class/html.formfile.class.php';
-if (!empty($conf->global->PRODUIT_PDF_MERGE_PROPAL)) {
+if (getDolGlobalString('PRODUIT_PDF_MERGE_PROPAL')) {
 	require_once DOL_DOCUMENT_ROOT.'/product/class/propalmergepdfproduct.class.php';
 }
 
+/**
+ * @var Conf $conf
+ * @var DoliDB $db
+ * @var HookManager $hookmanager
+ * @var Translate $langs
+ * @var User $user
+ */
 
 // Load translation files required by the page
 $langs->loadLangs(array('other', 'products'));
 
 
 // Get parameters
-$id     = GETPOST('id', 'int');
+$id     = GETPOSTINT('id');
 $ref    = GETPOST('ref', 'alpha');
 $action = GETPOST('action', 'aZ09');
 $confirm = GETPOST('confirm', 'alpha');
@@ -58,14 +68,14 @@ if ($user->socid) {
 	$socid = $user->socid;
 }
 
-// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
+// Initialize a technical object to manage hooks of page. Note that conf->hooks_modules contains an array of hook context
 $hookmanager->initHooks(array('productdocuments'));
 
 // Get parameters
-$limit = GETPOST('limit', 'int') ? GETPOST('limit', 'int') : $conf->liste_limit;
+$limit = GETPOSTINT('limit') ? GETPOSTINT('limit') : $conf->liste_limit;
 $sortfield = GETPOST('sortfield', 'aZ09comma');
 $sortorder = GETPOST('sortorder', 'aZ09comma');
-$page = GETPOSTISSET('pageplusone') ? (GETPOST('pageplusone') - 1) : GETPOST("page", 'int');
+$page = GETPOSTISSET('pageplusone') ? (GETPOSTINT('pageplusone') - 1) : GETPOSTINT("page");
 if (empty($page) || $page == -1) {
 	$page = 0;
 }     // If $page is not defined, or '' or -1
@@ -79,6 +89,8 @@ if (!$sortfield) {
 	$sortfield = "position_name";
 }
 
+$upload_dir = '';
+$upload_dirold = '';
 // Initialize objects
 $object = new Product($db);
 if ($id > 0 || !empty($ref)) {
@@ -90,7 +102,7 @@ if ($id > 0 || !empty($ref)) {
 		$upload_dir = $conf->service->multidir_output[$object->entity].'/'.get_exdir(0, 0, 0, 1, $object, 'product');
 	}
 
-	if (getDolGlobalInt('PRODUCT_USE_OLD_PATH_FOR_PHOTO')) {    // For backward compatiblity, we scan also old dirs
+	if (getDolGlobalInt('PRODUCT_USE_OLD_PATH_FOR_PHOTO')) {    // For backward compatibility, we scan also old dirs
 		if (isModEnabled("product")) {
 			$upload_dirold = $conf->product->multidir_output[$object->entity].'/'.substr(substr("000".$object->id, -2), 1, 1).'/'.substr(substr("000".$object->id, -2), 0, 1).'/'.$object->id."/photos";
 		} else {
@@ -120,7 +132,7 @@ $permissiontoadd = (($object->type == Product::TYPE_PRODUCT && $user->hasRight('
  * Actions
  */
 
-$parameters = array('id'=>$id);
+$parameters = array('id' => $id);
 $reshook = $hookmanager->executeHooks('doActions', $parameters, $object, $action); // Note that $action and $object may have been modified by some hooks
 if ($reshook < 0) {
 	setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');
@@ -128,7 +140,7 @@ if ($reshook < 0) {
 
 if (empty($reshook)) {
 	// Delete line if product propal merge is linked to a file
-	if (!empty($conf->global->PRODUIT_PDF_MERGE_PROPAL)) {
+	if (getDolGlobalString('PRODUIT_PDF_MERGE_PROPAL')) {
 		if ($action == 'confirm_deletefile' && $confirm == 'yes' && $permissiontoadd) {
 			//extract file name
 			$urlfile = GETPOST('urlfile', 'alpha');
@@ -154,6 +166,7 @@ if ($action == 'filemerge' && $permissiontoadd) {
 
 		$filetomerge_file_array = GETPOST('filetoadd');
 
+		$lang_id = null;
 		if (getDolGlobalInt('MAIN_MULTILANGS')) {
 			$lang_id = GETPOST('lang_id', 'aZ09');
 		}
@@ -161,7 +174,7 @@ if ($action == 'filemerge' && $permissiontoadd) {
 		// Delete all file already associated
 		$filetomerge = new Propalmergepdfproduct($db);
 
-		if (getDolGlobalInt('MAIN_MULTILANGS')) {
+		if (getDolGlobalInt('MAIN_MULTILANGS') && $lang_id !== null) {
 			$result = $filetomerge->delete_by_product($user, $object->id, $lang_id);
 		} else {
 			$result = $filetomerge->delete_by_product($user, $object->id);
@@ -176,7 +189,7 @@ if ($action == 'filemerge' && $permissiontoadd) {
 				$filetomerge->fk_product = $object->id;
 				$filetomerge->file_name = $filetomerge_file;
 
-				if (getDolGlobalInt('MAIN_MULTILANGS')) {
+				if (getDolGlobalInt('MAIN_MULTILANGS') && $lang_id !== null) {
 					$filetomerge->lang = $lang_id;
 				}
 
@@ -208,10 +221,10 @@ if (GETPOST("type") == '1' || ($object->type == Product::TYPE_SERVICE)) {
 	$helpurl = 'EN:Module_Services_En|FR:Module_Services|ES:M&oacute;dulo_Servicios';
 }
 
-llxHeader('', $title, $helpurl);
+llxHeader('', $title, $helpurl, '', 0, 0, '', '', '', 'mod-product page-card_document');
 
 
-if ($object->id) {
+if ($object->id > 0) {
 	$head = product_prepare_head($object);
 	$titre = $langs->trans("CardProduct".$object->type);
 	$picto = ($object->type == Product::TYPE_SERVICE ? 'service' : 'product');
@@ -226,10 +239,10 @@ if ($object->id) {
 	}
 
 	// Build file list
-	$filearray = dol_dir_list($upload_dir, "files", 0, '', '(\.meta|_preview.*\.png)$', $sortfield, (strtolower($sortorder) == 'desc' ?SORT_DESC:SORT_ASC), 1);
+	$filearray = dol_dir_list($upload_dir, "files", 0, '', '(\.meta|_preview.*\.png)$', $sortfield, (strtolower($sortorder) == 'desc' ? SORT_DESC : SORT_ASC), 1);
 
-	if (getDolGlobalInt('PRODUCT_USE_OLD_PATH_FOR_PHOTO')) {    // For backward compatiblity, we scan also old dirs
-		$filearrayold = dol_dir_list($upload_dirold, "files", 0, '', '(\.meta|_preview.*\.png)$', $sortfield, (strtolower($sortorder) == 'desc' ?SORT_DESC:SORT_ASC), 1);
+	if (getDolGlobalInt('PRODUCT_USE_OLD_PATH_FOR_PHOTO')) {    // For backward compatibility, we scan also old dirs
+		$filearrayold = dol_dir_list($upload_dirold, "files", 0, '', '(\.meta|_preview.*\.png)$', $sortfield, (strtolower($sortorder) == 'desc' ? SORT_DESC : SORT_ASC), 1);
 		$filearray = array_merge($filearray, $filearrayold);
 	}
 
@@ -240,10 +253,10 @@ if ($object->id) {
 
 
 	$linkback = '<a href="'.DOL_URL_ROOT.'/product/list.php?restore_lastsearch_values=1&type='.$object->type.'">'.$langs->trans("BackToList").'</a>';
-	$object->next_prev_filter = "fk_product_type = ".((int) $object->type);
+	$object->next_prev_filter = "(te.fk_product_type:=:".((int) $object->type).")";
 
 	$shownav = 1;
-	if ($user->socid && !in_array('product', explode(',', $conf->global->MAIN_MODULES_FOR_EXTERNAL))) {
+	if ($user->socid && !in_array('product', explode(',', getDolGlobalString('MAIN_MODULES_FOR_EXTERNAL')))) {
 		$shownav = 0;
 	}
 
@@ -263,12 +276,35 @@ if ($object->id) {
 
 	print dol_get_fiche_end();
 
+	/* To move into tpl
+	require_once DOL_DOCUMENT_ROOT.'/core/class/html.formmail.class.php';
+
+	$formmail = new FormMail($db);
+	$formmail->withaiprompt = 'text';
+	$out = '';
+
+	$showlinktoai = ($formmail->withaiprompt && isModEnabled('ai')) ? 'textgenerationemail' : '';
+	$showlinktoailabel = $langs->trans("GenerateImage");
+
+	$formatforouput = 'image';
+	$htmlname = 'bodyemail';
+
+	print load_fiche_titre($langs->trans('GenerateWithAI'), '', '');
+	print '<table class="border centpercent">';
+
+	// Fill $out
+	require DOL_DOCUMENT_ROOT.'/core/tpl/formlayoutai.tpl.php';
+
+	print $out;
+	print '</table>';
+	*/
+
 	$param = '&id='.$object->id;
 	include DOL_DOCUMENT_ROOT.'/core/tpl/document_actions_post_headers.tpl.php';
 
 
 	// Merge propal PDF document PDF files
-	if (!empty($conf->global->PRODUIT_PDF_MERGE_PROPAL)) {
+	if (getDolGlobalString('PRODUIT_PDF_MERGE_PROPAL')) {
 		$filetomerge = new Propalmergepdfproduct($db);
 
 		if (getDolGlobalInt('MAIN_MULTILANGS')) {
@@ -278,16 +314,13 @@ if ($object->id) {
 			$result = $filetomerge->fetch_by_product($object->id);
 		}
 
-		$form = new Form($db);
-
-
 		$filearray = dol_dir_list($upload_dir, "files", 0, '', '\.meta$', 'name', SORT_ASC, 1);
 
-		if (getDolGlobalInt('PRODUCT_USE_OLD_PATH_FOR_PHOTO')) {    // For backward compatiblity, we scan also old dirs
+		if (getDolGlobalInt('PRODUCT_USE_OLD_PATH_FOR_PHOTO')) {    // For backward compatibility, we scan also old dirs
 			$filearray = array_merge($filearray, dol_dir_list($upload_dirold, "files", 0, '', '\.meta$', 'name', SORT_ASC, 1));
 		}
 
-		// For each file build select list with PDF extention
+		// For each file build select list with PDF extension
 		if (count($filearray) > 0) {
 			print '<br>';
 			// Actual file to merge is :
@@ -304,6 +337,7 @@ if ($object->id) {
 
 			print  '<table class="noborder">';
 
+			$default_lang = null;
 			// Get language
 			if (getDolGlobalInt('MAIN_MULTILANGS')) {
 				$langs->load("languages");
@@ -317,18 +351,19 @@ if ($object->id) {
 				print Form::selectarray('lang_id', $langs_available, $default_lang, 0, 0, 0, '', 0, 0, 0, 'ASC');
 
 				if (getDolGlobalInt('MAIN_MULTILANGS')) {
-					print  '<input type="submit" class="button" name="refresh" value="'.$langs->trans('Refresh').'">';
+					print  '<input type="submit" class="button smallpaddingimp" name="refresh" value="'.$langs->trans('Refresh').'">';
 				}
 
 				print  '</td></tr>';
 			}
 
 			foreach ($filearray as $filetoadd) {
-				if ($ext = pathinfo($filetoadd['name'], PATHINFO_EXTENSION) == 'pdf') {
+				$ext = pathinfo($filetoadd['name'], PATHINFO_EXTENSION);
+				if ($ext == 'pdf') {
 					$checked = '';
 					$filename = $filetoadd['name'];
 
-					if (getDolGlobalInt('MAIN_MULTILANGS')) {
+					if (getDolGlobalInt('MAIN_MULTILANGS') && $default_lang !== null) {
 						if (array_key_exists($filetoadd['name'].'_'.$default_lang, $filetomerge->lines)) {
 							$filename = $filetoadd['name'].' - '.$langs->trans('Language_'.$default_lang);
 							$checked = ' checked ';
